/* ------------------------------------------------------------------------- */
/*
 *  constrained_value.h
 *
 *  Copyright (c) 2004 - 2007 "clown". All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Reference: D. Ryan Stephens, Christopher Diggins, Janathan Turkanis,
 *  and Jeff Cogswell, ``C++ Cookbook,'' O'Reilly Media, Inc, 2006.
 *
 *  Last-modified: Sun 11 Mar 2007 04:02:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_CONSTRAINED_VALUE_H
#define CLX_CONSTRAINED_VALUE_H

#include <cstdlib>
#include <iostream>
#include "constrained_value_func.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  constrained_value
	/* --------------------------------------------------------------------- */
	template <class ConstrainedFunc>
	class constrained_value {
	public:
		typedef ConstrainedFunc func;
		typedef typename ConstrainedFunc::value_type value_type;
		
		constrained_value() : value_(func::default_value) {}
		constrained_value(const constrained_value& x) : value_(x.value_) {}
		constrained_value(const value_type& x) { func::assign(value_, x); }
		virtual ~constrained_value() {}
		
		void assign(const value_type& x) {
			func::assign(value_, x);
		}
		
		value_type value() const { return value_; }
		
		// assignment operator
		constrained_value& operator=(const constrained_value& x) {
			this->assign(x.value_);
			return *this;
		}
		
		constrained_value& operator+=(const constrained_value& x) {
			this->assign(value_ + x.value_);
			return *this;
		}
		
		constrained_value& operator-=(const constrained_value& x) {
			this->assign(value_ - x.value_);
			return *this;
		}
		
		constrained_value& operator*=(const constrained_value& x) {
			this->assign(value_ * x.value_);
			return *this;
		}
		
		constrained_value& operator/=(const constrained_value& x) {
			this->assign(value_ / x.value_);
			return *this;
		}
		
		constrained_value& operator%=(const constrained_value& x) {
			this->assign(value_ % x.value_);
			return *this;
		}
		
		constrained_value& operator>>=(int x) {
			this->assign(value_ >> x);
			return *this;
		}
		
		constrained_value& operator<<=(int x) {
			this->assign(value_ << x);
			return *this;
		}
		
		// unary operator 
		constrained_value operator-() { return constrained_value(-value_); }
		constrained_value operator+() { return constrained_value(+value_); }
		constrained_value operator!() { return constrained_value(!value_); }
		constrained_value operator~() { return constrained_value(~value_); }
		
		// binary operator
		friend constrained_value operator+(constrained_value x, const value_type& y) { return x += y; }
		friend constrained_value operator-(constrained_value x, const value_type& y) { return x -= y; }
		friend constrained_value operator*(constrained_value x, const value_type& y) { return x *= y; }
		friend constrained_value operator/(constrained_value x, const value_type& y) { return x /= y; }
		friend constrained_value operator%(constrained_value x, const value_type& y) { return x %= y; }
		friend constrained_value operator+(const value_type& y, constrained_value x) { return x += y; }
		friend constrained_value operator-(const value_type& y, constrained_value x) { return x -= y; }
		friend constrained_value operator*(const value_type& y, constrained_value x) { return x *= y; }
		friend constrained_value operator/(const value_type& y, constrained_value x) { return x /= y; }
		friend constrained_value operator%(const value_type& y, constrained_value x) { return x %= y; }
		friend constrained_value operator>>(constrained_value x, int y) { return x >>= y; }
		friend constrained_value operator<<(constrained_value x, int y) { return x <<= y; }
		
		// stream operator
		template <class Ch, class Tr>
		friend std::basic_ostream<Ch, Tr>& operator<<(
			std::basic_ostream<Ch, Tr>& sout, constrained_value x) {
			sout << x.value_;
			return sout;
		}
		
		template <class Ch, class Tr>
		friend std::basic_istream<Ch, Tr>& operator>>(
			std::basic_istream<Ch, Tr>& sin, constrained_value x) {
			value_type tmp;
			sin >> tmp;
			x.assign(tmp);
			return sin;
		}
		
		// comparison operator
		friend bool operator<(const constrained_value& x, const constrained_value& y) { return x.value_ < y.value_; }
		friend bool operator>(const constrained_value& x, const constrained_value& y) { return x.value_ > y.value_; }
		friend bool operator<=(const constrained_value& x, const constrained_value& y) { return x.value_ <= y.value_; }
		friend bool operator>=(const constrained_value& x, const constrained_value& y) { return x.value_ >= y.value_; }
		friend bool operator==(const constrained_value& x, const constrained_value& y) { return x.value_ == y.value_; }
		friend bool operator!=(const constrained_value& x, const constrained_value& y) { return x.value_ != y.value_; }
		
	private:
		value_type value_;
	};
};

#endif // CLX_CONSTRAINED_VALUE_H
