/* ------------------------------------------------------------------------- */
/*
 *  inet_address.h
 *
 *  Copyright (c) 2004 - 2007 "clown". All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Wed 17 Jan 2007 12:02:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_INET_ADDRESS_H
#define CLX_INET_ADDRESS_H

#include <string>
#include <vector>
#include "socket.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  inet_address
	/* --------------------------------------------------------------------- */
	template <int Family = AF_INET>
	class basic_inet_address {
	public:
		typedef unsigned int size_type;
		typedef char char_type;
		typedef struct in_addr address_type;
		typedef std::basic_string<char_type> string_type;
		typedef std::vector<address_type> address_list;
		typedef std::vector<string_type> alias_list;
		
		// constructor and destructor
		basic_inet_address() : name_(), addr_(), aliases_(), len_() {}
		
		virtual ~basic_inet_address() {}
		
		basic_inet_address& operator=(const char_type* s) {
			this->lookup(s);
			return *this;
		}
		
		basic_inet_address& operator=(const string_type& s) {
			this->lookup(s);
			return *this;
		}
		
		// operator
		bool lookup(const char_type* s) {
			address_type addr;
			struct hostent* entry;
			
			if (!::inet_aton(s, &addr)) entry = ::gethostbyname(s);
			else entry = ::gethostbyaddr((const char*)&addr, sizeof(address_type), Family);
			
			if (entry == NULL) return false;
			name_ = entry->h_name;
			len_ = entry->h_length;
			for (int i = 0; entry->h_aliases[i]; i++) aliases_.push_back(entry->h_aliases[i]);
			for (int i = 0; entry->h_addr_list[i]; i++) addr_.push_back(*((address_type*)entry->h_addr_list[i]));
			
			return true;
		}
		
		
		bool lookup(const string_type& s) { return this->lookup(s.c_str()); }
		
		// get information about addresses
		bool empty() const { return addr_.empty(); }
		size_type size() const { return addr_.size(); }
		int length() const { return len_; }
		
		const address_type& at(size_type i) const { return addr_.at(i); }
		const address_type& operator[](size_type i) const { return addr_[i]; }
		
		string_type address_by_str(size_type i) {
			string_type tmp(::inet_ntoa(addr_.at(i)));
			return tmp;
		}
		
		// get information about hostname
		const string_type& hostname() const { return name_; }
		const alias_list& aliases() const { return aliases_; }
		
	private:
		string_type name_;
		address_list addr_;
		alias_list aliases_;
		int len_;
	};
	
	typedef basic_inet_address<> inet_address;
};

#endif // CLX_INET_ADDRESS_H
