/* ------------------------------------------------------------------------- */
/*
 *  quantile.h
 *
 *  Copyright (c) 2004 - 2007 "clown", All rights reserved.
 *  
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *  
 *  Last-modified: Tue 23 Jan 2007 00:16:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_QUANTILE_H
#define CLX_QUANTILE_H


#include <algorithm>
#include <vector>
#include <cmath>
#include "lexical_cast.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  quantile
	/* --------------------------------------------------------------------- */
	template <class InputIterator>
	double quantile(InputIterator first, InputIterator last, unsigned int k, unsigned int q) {
		if (k > q) return static_cast<double>(0);
		
		std::vector<double> v;
		while (first != last) v.push_back(lexical_cast<double>(*first++));
		std::sort(v.begin(), v.end());
		
		unsigned int idx = q + k * v.size() - k;
		if (idx % q == 0) return v.at(idx / q - 1);
		
		double t = idx / static_cast<double>(q);
		unsigned int ceil = static_cast<unsigned int>(std::ceil(t));
		unsigned int floor = static_cast<unsigned int>(std::floor(t));
		
		return (ceil - t) * v.at(floor - 1) + (t - floor) * v.at(ceil - 1);
	}
	
	/* --------------------------------------------------------------------- */
	//  median
	/* --------------------------------------------------------------------- */
	template <class InputIterator>
	double median(InputIterator first, InputIterator last) {
		return quantile(first, last, 1, 2);
	}
	
	/* --------------------------------------------------------------------- */
	//  tertile
	/* --------------------------------------------------------------------- */
	template <class InputIterator>
	double tertile(InputIterator first, InputIterator last, unsigned int k) {
		return quantile(first, last, k, 3);
	}
	
	/* --------------------------------------------------------------------- */
	//  quartile
	/* --------------------------------------------------------------------- */
	template <class InputIterator>
	double quartile(InputIterator first, InputIterator last, unsigned int k) {
		return quantile(first, last, k, 4);
	}
	
	/* --------------------------------------------------------------------- */
	//  quintile
	/* --------------------------------------------------------------------- */
	template <class InputIterator>
	double quintile(InputIterator first, InputIterator last, unsigned int k) {
		return quantile(first, last, k, 5);
	}
	
	/* --------------------------------------------------------------------- */
	//  decile
	/* --------------------------------------------------------------------- */
	template <class InputIterator>
	double decile(InputIterator first, InputIterator last, unsigned int k) {
		return quantile(first, last, k, 10);
	}
	
	/* --------------------------------------------------------------------- */
	//  percentile
	/* --------------------------------------------------------------------- */
	template <class InputIterator>
	double percentile(InputIterator first, InputIterator last, unsigned int k) {
		return quantile(first, last, k, 100);
	}
};

#endif // CLX_QUANTILE_H
