/* ------------------------------------------------------------------------- */
/*
 *  scanner.h
 *
 *  Copyright (c) 2004 - 2007 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Tue 16 Jan 2007 04:56:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_SCANNER_H
#define CLX_SCANNER_H

#include <istream>
#include <string>
#include "tokenizer.h"
#include "lexical_cast.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  basic_scanner
	/* --------------------------------------------------------------------- */
	template <
		class CharT,
		class Traits = std::char_traits<CharT>
	>
	class basic_scanner {
	public:
		typedef CharT char_type;
		typedef unsigned int size_type;
		typedef typename std::basic_string<CharT, Traits> string_type;
		typedef typename std::basic_istream<CharT, Traits> istream;
		
		explicit basic_scanner(istream& sin, const char_type* fmt) :
			f_(separator(fmt)) { this->scan(sin); }
		
		explicit basic_scanner(istream& sin, const string_type& fmt) :
			f_(separator(fmt)) { this->scan(sin); }
		
		explicit basic_scanner(const char_type* src, const char_type* fmt) :
			f_(separator(fmt)) { this->scan(src); }
		
		explicit basic_scanner(const string_type& src, const string_type& fmt) :
			f_(separator(fmt)) { this->scan(src); }
		
		~basic_scanner() {}
		
		template <class ValueT>
		basic_scanner& operator%(ValueT& dest) {
			return this->assign(dest);
		}
		
		template <class ValueT>
		basic_scanner& operator()(size_type pos, ValueT& dest) {
			return this->assign(pos, dest);
		}
		
		template <class ValueT, class Assign>
		basic_scanner& operator()(size_type pos, ValueT& dest, Assign f) {
			return this->assign(pos, dest, f);
		}
		
	private:
		typedef format_separator<CharT, Traits> separator;
		typedef basic_tokenizer<separator, std::vector<string_type> > functor;
		typedef typename functor::const_iterator const_iterator;
		
		functor f_;
		const_iterator cur_;
		
		void scan(const char_type* src) {
			f_.assign(src);
			cur_ = f_.begin();
		}
		
		void scan(const string_type& src) {
			this->scan(src.c_str());
		}
		
		void scan(istream& sin) {
			std::string tmp;
			std::getline(sin, tmp);
			this->scan(tmp);
		}
		
		template <class ValueT>
		basic_scanner& assign(ValueT& dest) {
			if (cur_ != f_.end()) {
				dest = lexical_cast<ValueT>(*cur_);
				cur_++;
			}
			return *this;
		}
		
		template <class ValueT>
		basic_scanner& assign(size_type pos, ValueT& dest) {
			if (pos < f_.size()) dest = lexical_cast<ValueT>(f_.at(pos));
			return *this;
		}
		
		template <class ValueT, class Assign>
		basic_scanner& assign(size_type pos, ValueT& dest, Assign f) {
			if (pos < f_.size()) f(f_.at(pos), dest);
			return *this;
		}
	};
	
	typedef basic_scanner<char> scanner;
}

#endif // CLX_SCANNER_H
