/* ------------------------------------------------------------------------- */
/*
 *  table.h
 *
 *  Copyright (c) 2004 - 2007 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Tue 16 Jan 2007 04:55:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_TABLE_H
#define CLX_TABLE_H

#include <string>
#include <vector>
#include <fstream>

namespace clx {
	/* --------------------------------------------------------------------- */
	//  table
	/* --------------------------------------------------------------------- */
	template <
		class Parser,
		class Container = std::vector<typename Parser::container>,
		class CharT = char,
		class Traits = std::char_traits<CharT>
	>
	class table : public Container {
	public:
		typedef CharT char_type;
		typedef Parser functor;
		typedef Container container;
		typedef typename functor::value_type value_type;
		typedef typename container::size_type size_type;
		typedef std::basic_string<CharT, Traits> string_type;
		typedef std::basic_fstream<CharT, Traits> fstream;
		
		// constructor and destructor
		explicit table(const functor& f = functor()) :
			container(), fp_(), f_(f) { this->reset(); }
		
		explicit table(const char_type* path, const functor& f = functor()) :
			container(), fp_(), f_(f) {
			this->reset();
			this->open(path);
		}
		
		explicit table(const string_type& path, const functor& f = functor()) :
			container(), fp_(), f_(f) {
			this->reset();
			this->open(path);
		}
		
		virtual ~table() { if (fp_.is_open()) fp_.close(); }
		
		// operator
		virtual void reset() {
			this->clear();
			f_.reset();
		}
		
		bool readline() {
			string_type tmp;
			if (!std::getline(fp_, tmp)) return false;
			this->insert(this->end(), f_(tmp));
			return true;
		}
		
		table& read(size_type line) {
			for (size_type i = line; i > 0; i--) {
				if (!this->readline()) break;
			}
			return *this;
		}
		
		table& read() {
			while (this->readline());
			return *this;
		}
		
		void open(const char_type* path) { fp_.open(path); }
		void open(const string_type& path) { this->open(path.c_str()); }
		void close() { fp_.close(); }
		bool is_open() { return fp_.is_open(); }
		bool eof() { return fp_.eof(); }
		
	private:
		fstream fp_;
		functor f_;
		
		// invalid copy constructor and self-assign operator
		explicit table(const table& cp) {
			throw std::runtime_error("Operation is not permitted");
		}
		
		table& operator=(const table& cp) {
			throw std::runtime_error("Operation is not permitted");
		}
	};
}

#endif // CLX_TABLE_H
