/* ------------------------------------------------------------------------- */
/*
 *  tcpsocket.h
 *
 *  Copyright (c) 2004 - 2007 "clown". All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Fri 16 Mar 2007 17:52:03 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_TCPSOCKET_H
#define CLX_TCPSOCKET_H

#include <string>
#include <sstream>
#include "socket.h"
#include "sockaddress.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  basic_serversocket
	/* --------------------------------------------------------------------- */
	template <
		int Type = SOCK_STREAM,
		int Family = AF_INET
	>
	class basic_serversocket : public basic_rawsocket<Type, Family> {
	public:
		typedef basic_rawsocket<Type, Family> rawsocket;
		typedef basic_sockaddress<Family> sockaddress;
		
		// constructor and destructor
		basic_serversocket() : rawsocket() {}
		
		explicit basic_serversocket(int port, int n = 100) :
			rawsocket() { this->open(port, n); }
		
		virtual ~basic_serversocket() {}
		
		// operator
		socket_type open(int port, int n = 100) {
			if (rawsocket::open() < 0) throw socket_error("socket");
			addr_.assign(port);
			if (::bind(this->socket(), addr_.base_data(), addr_.size()) == -1) {
				throw socket_error("bind");
			}
			if (::listen(this->socket(), n) == -1) throw socket_error("listen");
			return this->socket();
		}
		
		socket_type accept(sockaddress& addr, int retry = 10) {
			socket_type peer = -1;
			typename sockaddress::inet_type tmp;
			socklen_type len = sizeof(tmp);
			
			for (int i = 0; i < retry; i++) {
				peer = ::accept(this->socket(), (struct sockaddr*)&tmp, &len);
				if (peer == -1) ::sleep(1);
				else break;
			}
			
			addr = tmp;
			return peer;
		}
		
		socket_type accept(int retry = 10) {
			sockaddress tmp;
			return this->accept(tmp, retry);
		}
	
	private:
		sockaddress addr_;
	};
	
	/* --------------------------------------------------------------------- */
	//  basic_clientsocket
	/* --------------------------------------------------------------------- */
	template <
		int Type = SOCK_STREAM,
		int Family = AF_INET
	>
	class basic_clientsocket : public basic_rawsocket<Type, Family> {
	public:
		typedef char char_type;
		typedef basic_rawsocket<Type, Family> rawsocket;
		typedef basic_sockaddress<Family> sockaddress;
		typedef typename std::basic_string<char_type> string_type;
		
		// constructor and destructor
		basic_clientsocket() : rawsocket(), addr_() {}
		
		explicit basic_clientsocket(const char_type* host, int port) :
			rawsocket(), addr_() { this->connect(host, port); }
		
		explicit basic_clientsocket(const string_type& host, int port) :
			rawsocket(), addr_() { this->connect(host, port); }
		
		virtual ~basic_clientsocket() {}
		
		// operator
		socket_type connect(const char_type* host, int port) {
			if (this->open() < 0) throw socket_error("socket");
			addr_.assign(host, port);
			if (::connect(this->socket(), addr_.base_data(), addr_.size()) == -1) {
				throw socket_error("connect");
			}
			return this->socket();
		}
		
		socket_type connect (const string_type& host, int port) {
			return this->connect(host.c_str(), port);
		}
		
	private:
		sockaddress addr_;
	};
	
	typedef basic_serversocket<> serversocket;
	typedef basic_clientsocket<> clientsocket;
}

#endif // CLX_TCPSOCKET_H
