/* ------------------------------------------------------------------------- */
/*
 *  tokenizer.h
 *
 *  Copyright (c) 2004 - 2007 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Tue 16 Jan 2007 04:56:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_TOKENIZER_H
#define CLX_TOKENIZER_H

#include <string>
#include <vector>
#include "tokenizer_func.h"
#include "lexical_cast.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  basic_tokenizer
	/* --------------------------------------------------------------------- */
	template <
		class TokenizerFunc,
		class Container,
		class CharT = char,
		class Traits = std::char_traits<CharT>
	>
	class basic_tokenizer : public Container {
	public:
		typedef CharT char_type;
		typedef TokenizerFunc functor;
		typedef Container container;
		typedef std::basic_string<CharT, Traits> string_type;
		typedef typename container::size_type size_type;
		typedef typename container::value_type value_type;
		
		// constructor and destructor
		explicit basic_tokenizer(const functor& f = functor()) :
			super(), f_(f) {}
		
		template <class InputIterator>
		explicit basic_tokenizer(InputIterator first, InputIterator last, const functor& f = functor()) :
			super(), f_(f) { this->assign(first, last); }
		
		explicit basic_tokenizer(const string_type& s, const functor& f = functor()) :
			super(), f_(f) { this->assign(s); }
		
		explicit basic_tokenizer(const char_type* s, const functor& f = functor()) :
			super(), f_(f) { this->assign(s); }
		
		virtual ~basic_tokenizer() {}
		
		//operator
		virtual void reset() {
			f_.reset();
			this->clear();
		}
		
		template <class InputIterator>
		basic_tokenizer& assign(InputIterator first, InputIterator last) {
			string_type token;
			
			this->reset();
			while (f_(first, last, token)) {
				this->insert(this->end(), lexical_cast<value_type>(token));
				if (first++ == last) break;
			}
			return *this;
		}
		
		basic_tokenizer& assign(const string_type& s) {
			typename string_type::const_iterator pos = s.begin();
			return this->assign(pos, s.end());
		}
		
		basic_tokenizer& assign(const char_type* s) {
			string_type tmp(s);
			return this->assign(tmp);
		}
		
		template <class InputIterator>
		basic_tokenizer& operator()(InputIterator first, InputIterator last) {
			return this->assign(first, last);
		}
		
		basic_tokenizer& operator()(const string_type& s) { return this->assign(s); }
		basic_tokenizer& operator()(const char_type* s) { return this->assign(s); }
		
	private:
		typedef Container super;
		
		functor f_;
	};
	
	typedef basic_tokenizer<char_separator<char>, std::vector<std::string> > tokenizer;
	typedef basic_tokenizer<escape_separator<char>, std::vector<std::string> > csv;
	typedef basic_tokenizer<format_separator<char>, std::vector<std::string> > strftokenizer;
}

#endif // CLX_TOKENIZER_H
