/* ------------------------------------------------------------------------- */
/*
 *  tokenmap.h
 *
 *  Copyright (c) 2004 - 2007 "clown", All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Fri 16 Mar 2007 23:31:14 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_TOKENMAP_H
#define CLX_TOKENMAP_H

#include <string>
#include <map>
#include "tokenizer_func.h"
#include "lexical_cast.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  basic_tokenmap
	/* --------------------------------------------------------------------- */
	template <
		class TokenizerFunc,
		class Container,
		class CharT = char,
		class Traits = std::char_traits<CharT>
	>
	class basic_tokenmap : public Container {
	public:
		typedef CharT char_type;
		typedef TokenizerFunc functor;
		typedef Container container;
		typedef std::basic_string<CharT, Traits> string_type;
		typedef typename container::key_type key_type;
		typedef typename container::mapped_type mapped_type;
		typedef typename container::value_type value_type;
		
		// constructor and destructor
		explicit basic_tokenmap(const functor& f = functor()) :
			super(), f_(f) {}
		
		template <class InputIterator>
		explicit basic_tokenmap(InputIterator first, InputIterator last, const functor& f = functor()) :
			super(), f_(f) { this->assign(first, last); }
		
		explicit basic_tokenmap(const string_type& s, const functor& f = functor()) :
			super(), f_(f) { this->assign(s); }
		
		explicit basic_tokenmap(const char_type* s, const functor& f = functor()) :
			super(), f_(f) { this->assign(s); }
		
		virtual ~basic_tokenmap() {}
		
		//operator
		virtual void reset() {
			f_.reset();
			this->clear();
		}
		
		template <class InputIterator>
		basic_tokenmap& assign(InputIterator first, InputIterator last) {
			string_type tmp;
			string_type token;
			
			this->reset();
			while (f_(first, last, tmp, token)) {
				key_type key = lexical_cast<key_type>(tmp);
				mapped_type val = lexical_cast<mapped_type>(token);
				this->insert(value_type(key, val));
				if (first++ == last) break;
			}
			return *this;
		}
		
		basic_tokenmap& assign(const string_type& s) {
			typename string_type::const_iterator pos = s.begin();
			return this->assign(pos, s.end());
		}
		
		basic_tokenmap& assign(const char_type* s) {
			string_type tmp(s);
			return this->assign(tmp);
		}
		
		template <class InputIterator>
		basic_tokenmap& operator()(InputIterator first, InputIterator last) {
			return this->assign(first, last);
		}
		
		basic_tokenmap& operator()(const string_type& s) { return this->assign(s); }
		basic_tokenmap& operator()(const char_type* s) { return this->assign(s); }
		
	private:
		typedef Container super;
		
		functor f_;
	};
	
	typedef basic_tokenmap<format_separator<char>, std::map<char, std::string> > tokenmap;
}

#endif // CLX_TOKENMAP_H
