/* ------------------------------------------------------------------------- */
/*
 *  udpsocket.h
 *
 *  Copyright (c) 2004 - 2006 "clown". All rights reserved.
 *
 *  Permission to copy, use, modify and distribute this software is
 *  granted provided this copyright notice appears in all copies.
 *  This software is provided "as is" without express or implied
 *  warranty, and with no claim as to its suitability for any purpose.
 *
 *  Last-modified: Wed 06 Sep 2006 23:55:00 JST
 */
/* ------------------------------------------------------------------------- */
#ifndef CLX_UDPSOCKET_H
#define CLX_UDPSOCKET_H

#include <cstdlib>
#include <cstring>
#include <string>
#include "socket.h"
#include "sockaddress.h"

namespace clx {
	/* --------------------------------------------------------------------- */
	//  basic_udpsocket
	/* --------------------------------------------------------------------- */
	template <
		int Type = SOCK_DGRAM,
		int Family = AF_INET
	>
	class basic_udpsocket : basic_rawsocket<Type, Family> {
	public:
		typedef basic_sockaddress<Family> sockaddress;
		typedef char char_type;
		typedef std::basic_string<char_type> string_type;
		typedef unsigned int size_type;
		
		static const size_type bufsize = 4096;
		
		// constructor and destructor
		basic_udpsocket() :
			basic_rawsocket<Type, Family>(), is_bind_(false) {
			buf_ = new char_type[bufsize];
			this->open();
		}
		
		explicit basic_udpsocket(int port) :
			basic_rawsocket<Type, Family>(), is_bind_(false) { 
			buf_ = new char_type[bufsize];
			this->open();
			this->bind(port);
		}
		
		virtual ~basic_udpsocket() {
			delete buf_;
		}
		
		// operator
		int send(const char_type* src, const sockaddress& addr) {
			string_type tmp(src);
			return this->send(src, addr);
		}
		
		int send(const string_type& src, const sockaddress& addr) {
			return ::sendto(this->socket(), src.c_str(), src.size(), 0, addr.base_data(), addr.size());
		}
		
		int send(const char_type* src, const char_type* host, int port) {
			sockaddress addr(host, port);
			return this->send(src, addr);
		}
		
		int send(const string_type& src, const string_type& host, int port) {
			sockaddress addr(host, port);
			return this->send(src, addr);
		}
		
		int recv(string_type& dest, sockaddress& addr) {
			if (!this->is_bind()) return -1;
			
			int len, n;
			typename sockaddress::inet_type tmp;
			
			len = sizeof(tmp);
			std::memset(buf_, 0, bufsize);
			n = ::recvfrom(this->socket(), buf_, bufsize, 0, (struct sockaddr*)&tmp, &len);
			
			if (n > 0) dest = buf_;
			addr = tmp;
			return n;
		}
		
		int recv(string_type& dest) {
			sockaddress tmp;
			return this->recv(dest, tmp);
		}
		
		int bind(int port) {
			addr_.assign(port);
			if (::bind(this->socket(), addr_.base_data(), addr_.size()) == -1) return -1;
			is_bind_ = true;
			return 0;
		}
		
		bool is_bind() const { return is_bind_; }
		
	private:
		sockaddress addr_;
		char_type* buf_;
		bool is_bind_;
	};
	
	typedef basic_udpsocket<> udpsocket;
}

#endif // CLX_UDPSOCKET_H
